#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai


__license__   = 'GPL v3'
__copyright__ = '2021, Bradley Lang'
__docformat__ = 'restructuredtext en'

try:
    load_translations()
except NameError:
    pass # load_translations() added in calibre 1.9

import logging
logger = logging.getLogger(__name__)

try:
    from qt.core import Qt, QTableWidget, QAbstractItemView, QTableWidgetItem, QColor
except ImportError:
    from PyQt5.Qt import Qt, QTableWidget, QAbstractItemView, QTableWidgetItem, QColor

from calibre_plugins.librarything_match.config import prefs

colour_dict = {
                "green" :   QColor(0, 255, 0, 128),
                "yellow":   QColor(255, 255, 0, 128),
                "red"   :   QColor(255, 0, 0, 128),
                "none"  :   QColor(0, 0, 0, 0)
              }
        

class LTMatchBookWidget(QTableWidgetItem):

    # the class supports two columns, without having to manage them individually.
    # the idea is that this could be extended to use an arbitrary number of columns 
    # using Arbitrary Arguments (*args) without having to change the calling code.
    def __init__(self, author, title, index = -1, id=None):

        if author is None:
            author = ''
        #QTableWidgetItem.__init__(self, author, QTableWidgetItem.UserType)
        #QTableWidgetItem.__init__(self, author, 1000)
        QTableWidgetItem.__init__(self, author)
        
        # this object also owns one other qTW object (or multiple) for the other columns.
        self.title = QTableWidgetItem(title)

        if index != -1:
            self.setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled)
            self.setToolTip(author)
            self.title.setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled)
            self.title.setToolTip(title)
            
        else:
            self.setFlags(Qt.NoItemFlags)
            self.title.setFlags(Qt.NoItemFlags)

        # id can be a LibraryThing ID (text) or a calibre id (int). We don't really care here.
        # index is the location in the list, which each item has to remember.
        self.id = id
        self.index = index


    def getID(self):
        return self.id


    def getListIndex(self):
        return self.index


    def setListIndex(self, index):
        self.index = index


    def update_item(self, tbl):
        # LTMBW knows how to update a LTMTW/qTW with itself
        # we say "update" because the row always already exists
        tbl.setItem(self.index, 0, self)
        tbl.setItem(self.index, 1, self.title)


    def remove_item(self, tbl, row):
        # LTMBW knows how to remove itself from a LTMTW/qTW
        tbl.takeItem(row, 0)
        self.title = tbl.takeItem(row, 1)
        return self


    def setDisabled(self):
        
        self.setFlags(Qt.NoItemFlags)
        self.title.setFlags(Qt.NoItemFlags)
        italic = self.font()
        italic.setItalic(True)
        self.setFont(italic)
        self.title.setFont(italic)


class LTMatchStatusWidget(QTableWidgetItem):

    # this lightweight class is simply for showing the matched status in the status table.
    def __init__(self):

        #QTableWidgetItem.__init__(self, QTableWidgetItem.UserType)
        #QTableWidgetItem.__init__(self, 1000)
        QTableWidgetItem.__init__(self)

        self.setFlags(Qt.ItemIsEnabled)
        self.setTextAlignment(Qt.AlignCenter)


    def setMatchStatus(self, match_status, percent=100):

        threshold = int(prefs['probable_match_threshold'])

        if match_status == _("Matched"):
            self.setText(_("Matched"))
            self.setBackground(colour_dict["green"])
        elif match_status == _("Partial"):
            self.setText("{:.0%}".format(percent/100))
            if percent >= threshold:
                self.setBackground(colour_dict["green"])
            elif percent >= 50 and percent < threshold:
                self.setBackground(colour_dict["yellow"])
            elif percent >= 10 and percent < 50:
                self.setBackground(colour_dict["red"])
            else:
                self.setBackground(colour_dict["none"])
        else:
            self.setBackground(colour_dict["none"])


class LTMatchTableWidget(QTableWidget):

    def __init__(self, parent):

        QTableWidget.__init__(self, parent)
        # highlight selected row even when it doesn't have the focus.
        p = self.palette()
        p.setBrush(p.Inactive, p.Highlight, p.brush(p.Highlight))
        p.setBrush(p.Inactive, p.HighlightedText, p.brush(p.HighlightedText))
        self.setPalette(p)

        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setSelectionMode(QAbstractItemView.SingleSelection)
        self.setEditTriggers(QTableWidget.NoEditTriggers)
        self.setSortingEnabled(False)
        self.setAlternatingRowColors(True)
        header_labels = [_("Author"),_("Title")]
        self.setColumnCount(len(header_labels))
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setHorizontalHeaderLabels(header_labels)
        self.horizontalHeader().setStretchLastSection(True)
        self.verticalHeader().hide()
        self.setMinimumColumnWidth(0, 150)

        self.gui = parent.gui


    def setMinimumColumnWidth(self, col, minimum):
        if self.columnWidth(col) < minimum:
            self.setColumnWidth(col, minimum)


    def getFirstEmptyRow(self, row=0):
        rows = self.rowCount()
        for i in range(row+1,rows):
            itm = self.item(i, 0)
            if not itm:
                break
        return i


    def get_selected_book(self):
        # selectedRows() returns a list of QModelIndex objects.
        # return None if selection is invalid so calling function can chastise user.
        row = self.selectionModel().selectedRows()
        if len(row) == 1:
            itm = self.item(row[0].row(), 0)
            return itm


    def get_selected_row(self):
        # selectedRows() returns a list of QModelIndex objects.
        idx = -1
        row = self.selectionModel().selectedRows()
        if len(row) == 1:
            idx = row[0].row()
            return idx


    def remove_item(self, row):
        # call this instead of takeItem(). LTMTW calls LTMBW to remove itself from table.
        itm = self.item(row, 0)
        if itm is not None:
            itm = itm.remove_item(self, row)
        return itm